from alnsoft.structs.phylo_tree import PhyloTreeNode, PhyloTree
from alnsoft.utils.camber_phylo_utils import getTreeStrainList

def splitStrSubtrees(strains_str):
    strains_strs = []
    curr_str = ""
    count = 0
    for i in range(0, len(strains_str), 1):
        if(strains_str[i]=="("):
            curr_str += strains_str[i]
            count += 1
        elif(strains_str[i]==")"):
            curr_str += strains_str[i]
            count -= 1
        elif(count == 0 and strains_str[i]==","):
            strains_strs.append(curr_str)
            curr_str = ""
        else:
            curr_str += strains_str[i]
    strains_strs.append(curr_str)
    
    return strains_strs

def readPhyloTreeRec(tree, tree_str, strains_list, strains_map={}, e_len=None):
    last_index = tree_str.rfind(")")
    if last_index == -1:
        tokens = tree_str.split(":")
        strain_name = tokens[0]
        strain_index = strains_list.index(strain_name)
        strain_label = strains_map.get(strain_name, strain_name)
        strain_node = PhyloTreeNode(strain_index)
        strain_node.setLabel(strain_label)
        if len(tokens) == 2 and e_len == None:
            strain_node.e_len = float(tokens[1])
        elif e_len == None:
            strain_node.e_len = 1.0
        else:
            strain_node.e_len = e_len
        tree.addLeafNode(strain_node)
        return strain_node
    else:
        node_id = tree.createNodeID("")
        node = PhyloTreeNode(node_id)
        if last_index + 2 < len(tree_str):
            len_index = tree_str.rfind(":")
            if e_len == None and len_index > 0:
                node.e_len = float(tree_str[len_index+1:])
            elif e_len == None:
                node.e_len = 1.0
            else:
                node.e_len = e_len
        tree.addInternalNode(node)
        subtrees_str = splitStrSubtrees(tree_str[1:last_index])
        
        for subtree_str in subtrees_str:
            subtree_node = readPhyloTreeRec(tree, subtree_str, strains_list, strains_map, e_len)
            node.addChildren(subtree_node)
            subtree_node.parent = node
        return node

def readPhyloTree(tree_text, strains_list=None, strains_map={}, e_len=None):
    if strains_list == None:
        strains_list = getTreeStrainList(tree_text)
          
    tree = PhyloTree(len(strains_list)) 
    root = readPhyloTreeRec(tree, tree_text, strains_list, strains_map, e_len)
    tree.setRoot(root)

    tree.setLeavesByIDs(range(tree.leaves_count))
    return tree
    